#version 120
#extension GL_ARB_shader_texture_lod : enable

/*
Before modify anything, read the agreement first that can be found here : https://werrusshaders.wordpress.com/download/
Werrus shaders, derived from Chocapic13 V5.
This code is from Chocapic13' shaders.
Disable an effect by putting "//" before "#define" when there is no number after.
*/

//////////////////////////////ADJUSTABLE VARIABLES//////////////////////////////

	#define SHADOW								//enable shadow
	const int shadowMapResolution = 1536;		//shadow resolution
	const float	shadowDistance = 150.0;			//draw distance of shadows
	#define SHADOW_DARKNESS 0.7					//shadow darkness levels, lower values mean darker shadows, see .vsh for colors /0.25 is default
	#define SHADOW_FILTER						//smooth shadow
	//#define WIP_VARIABLE_PENUMBRA_SHADOWS		//explained, requires a powerful pc to run this! (not compatible with SHADOW_FILTER)
	//#define FAKE_SHADOW						//enable fake shadow

	#define DYNAMIC_HANDLIGHT					//emitting light around player when holding light-emitting item
	
	#define SUNLIGHTAMOUNT 1.9					//change sunlight strength , see .vsh for colors. /1.7 is default
	
	#define TORCH_COLOR_LIGHTING 1.0,0.5,0.1 	//Torch Color RGB - Red, Green, Blue / vec3(0.6,0.32,0.1) is default
	#define TORCH_ATTEN 5.0						//how much the first torch light will be attenuated (decrease if you want the torches to cover a bigger area))/3.0 is default
	#define TORCH_INTENSITY 0.925				//first torch light intensity /2.6 is default
	#define TORCH_MULT 1.3						//First Torch lightmap attenuation function, where x is torch light from minecraft lightmap  : ((x*mult)^atten)*intensity
	#define TORCH_ATTEN2 1.0					//how much the second torch light will be attenuated (decrease if you want the torches to cover a bigger area))/3.0 is default
	#define TORCH_INTENSITY2 0.225				//second torch light intensity /2.6 is default
	#define TORCH_MULT2 1.0						//Second Torch lightmap attenuation function, where x is torch light from minecraft lightmap  : ((x*mult)^atten)*intensity

	#define ATTENUATION 1.45					//reduce lightmap distance on shaded area(default setting = with shadow : 1.45 , without shadow : 3.0)
	#define MIN_LIGHT 0.01						//minimal of the light

	//#define SSAO								//enable ssao effect

	//#define CELSHADING						//cartoon-like effect

	#define WATER_REFRACT						//enable underwater movement
	#define WATER_CAUSTIC						//enable water light effect

//////////////////////////////END OF ADJUSTABLE VARIABLES///////////////////////

const bool 		shadowtex1Mipmap 		= true;
const bool 		shadowtex1Nearest 		= true;
const float 	wetnessHalflife 		= 70.0f;
const float 	drynessHalflife 		= 70.0f;
const float		eyeBrightnessHalflife	= 5.0f;
const int 		noiseTextureResolution  = 1024;
const float 	ambientOcclusionLevel	= 1.0;
const float		sunPathRotation			= -40.0f;

#ifdef SHADOW
const bool 		shadowHardwareFiltering = true;
#endif

varying vec4 texcoord;

varying vec3 lightVector;
varying vec3 sunVec;
varying vec3 moonVec;
varying vec3 upVec;

varying vec3 sunlight;
varying vec3 moonlight;
varying vec3 ambient_color;

varying vec4 lightS;

varying float handItemLight;
varying float eyeAdapt;

varying float SdotU;
varying float MdotU;
varying float sunVisibility;
varying float moonVisibility;

uniform sampler2D gcolor;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D gnormal;
uniform sampler2D gdepth;

#ifdef SHADOW
uniform sampler2DShadow shadow;
#endif
uniform sampler2D shadowtex1;

uniform sampler2D gaux1;
uniform sampler2D gaux2;
uniform sampler2D noisetex;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferModelView;
uniform mat4 shadowProjection;
uniform mat4 shadowModelView;
uniform mat4 shadowModelViewInverse;

uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform vec3 upPosition;
uniform vec3 cameraPosition;

uniform float near;
uniform float far;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float wetness;
uniform float aspectRatio;
uniform float frameTimeCounter;
uniform ivec2 eyeBrightness;
uniform ivec2 eyeBrightnessSmooth;
uniform int isEyeInWater;
uniform int worldTime;
uniform int fogMode;
float timefract = worldTime;

#define SHADOW_MAP_BIAS 0.85
#define MAX_COLOR_RANGE 48.0

//Calculate Time of Day
float TimeSunrise  = ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(timefract, 0.0, 4000.0)/4000.0));
float TimeNoon     = ((clamp(timefract, 0.0, 4000.0)) / 4000.0) - ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0);
float TimeSunset   = ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0) - ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0);
float TimeMidnight = ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0) - ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0);

vec3 convertScreenSpaceToWorldSpace(vec2 co, float depth) {
    vec4 fragposition = gbufferProjectionInverse * vec4(vec3(co, depth) * 2.0 - 1.0, 1.0);
    fragposition /= fragposition.w;
    return fragposition.xyz;
}

vec3 convertCameraSpaceToScreenSpace(vec3 cameraSpace) {
    vec4 clipSpace = gbufferProjection * vec4(cameraSpace, 1.0);
    vec3 NDCSpace = clipSpace.xyz / clipSpace.w;
    vec3 screenSpace = 0.5 * NDCSpace + 0.5;
    return screenSpace;
}

float luma(vec3 color) {
	return dot(color,vec3(0.299, 0.587, 0.114));
}

float ld(float depth) {
    return (2.0 * near) / (far + near - depth * (far - near));
}

vec3 nvec3(vec4 pos) {
    return pos.xyz/pos.w;
}

vec4 nvec4(vec3 pos) {
    return vec4(pos.xyz, 1.0);
}

float edepth(vec2 coord) {
	return texture2D(depthtex0,coord).z;
}
vec2 newtc = texcoord.xy;
vec3 sky_color = normalize(vec3(0.1, 0.35, 1.));

float pw = 1.0/ viewWidth;
float ph = 1.0/ viewHeight;

vec3 aux = texture2D(gaux1, texcoord.st).rgb;
vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0f - 1.0f;


float pixeldepth = texture2D(depthtex0,texcoord.xy).x;
float handlight = handItemLight;

float torch_lightmap = pow(max(aux.b,0.0)*TORCH_MULT,TORCH_ATTEN)*TORCH_INTENSITY;
float torch_lightmap2 = pow(max(aux.b,0.0)*TORCH_MULT2,TORCH_ATTEN2)*TORCH_INTENSITY2;

float sky_lightmap = pow(aux.r,(ATTENUATION-isEyeInWater*0.75));

float iswet = wetness*pow(sky_lightmap,5.0)*sqrt(0.5+max(dot(normal,upVec),0.0));

float isshaded = pow(sky_lightmap,3.0)*sqrt(0.5+max(dot(normal,upVec),0.0));
	
//poisson distribution for shadow sampling		
	const vec2 shadow_offsets[60] = vec2[60]  (  vec2( 0.0000, 0.2500 ),
									vec2( -0.2165, 0.1250 ),
									vec2( -0.2165, -0.1250 ),
									vec2( -0.0000, -0.2500 ),
									vec2( 0.2165, -0.1250 ),
									vec2( 0.2165, 0.1250 ),
									vec2( 0.0000, 0.5000 ),
									vec2( -0.2500, 0.4330 ),
									vec2( -0.4330, 0.2500 ),
									vec2( -0.5000, 0.0000 ),
									vec2( -0.4330, -0.2500 ),
									vec2( -0.2500, -0.4330 ),
									vec2( -0.0000, -0.5000 ),
									vec2( 0.2500, -0.4330 ),
									vec2( 0.4330, -0.2500 ),
									vec2( 0.5000, -0.0000 ),
									vec2( 0.4330, 0.2500 ),
									vec2( 0.2500, 0.4330 ),
									vec2( 0.0000, 0.7500 ),
									vec2( -0.2565, 0.7048 ),
									vec2( -0.4821, 0.5745 ),
									vec2( -0.6495, 0.3750 ),
									vec2( -0.7386, 0.1302 ),
									vec2( -0.7386, -0.1302 ),
									vec2( -0.6495, -0.3750 ),
									vec2( -0.4821, -0.5745 ),
									vec2( -0.2565, -0.7048 ),
									vec2( -0.0000, -0.7500 ),
									vec2( 0.2565, -0.7048 ),
									vec2( 0.4821, -0.5745 ),
									vec2( 0.6495, -0.3750 ),
									vec2( 0.7386, -0.1302 ),
									vec2( 0.7386, 0.1302 ),
									vec2( 0.6495, 0.3750 ),
									vec2( 0.4821, 0.5745 ),
									vec2( 0.2565, 0.7048 ),
									vec2( 0.0000, 1.0000 ),
									vec2( -0.2588, 0.9659 ),
									vec2( -0.5000, 0.8660 ),
									vec2( -0.7071, 0.7071 ),
									vec2( -0.8660, 0.5000 ),
									vec2( -0.9659, 0.2588 ),
									vec2( -1.0000, 0.0000 ),
									vec2( -0.9659, -0.2588 ),
									vec2( -0.8660, -0.5000 ),
									vec2( -0.7071, -0.7071 ),
									vec2( -0.5000, -0.8660 ),
									vec2( -0.2588, -0.9659 ),
									vec2( -0.0000, -1.0000 ),
									vec2( 0.2588, -0.9659 ),
									vec2( 0.5000, -0.8660 ),
									vec2( 0.7071, -0.7071 ),
									vec2( 0.8660, -0.5000 ),
									vec2( 0.9659, -0.2588 ),
									vec2( 1.0000, -0.0000 ),
									vec2( 0.9659, 0.2588 ),
									vec2( 0.8660, 0.5000 ),
									vec2( 0.7071, 0.7071 ),
									vec2( 0.5000, 0.8660 ),
									vec2( 0.2588, 0.9659 ));
									

float Blinn_Phong(vec3 ppos, vec3 lvector, vec3 normal,float fpow, float gloss, float visibility)  {
	vec3 lightDir = vec3(lvector);
	
	vec3 surfaceNormal = normal;
	float cosAngIncidence = dot(surfaceNormal, lightDir);
	cosAngIncidence = clamp(cosAngIncidence, 0.0, 1.0);
	
	vec3 viewDirection = normalize(-ppos);
	
	vec3 halfAngle = normalize(lightDir + viewDirection);
	float blinnTerm = dot(surfaceNormal, halfAngle);
	
	float normalDotEye = dot(normal, normalize(ppos));
	float fresnel = clamp(pow(1.0 + normalDotEye, 5.0),0.0,1.0);
	fresnel = fresnel*0.85 + 0.15 * (1.0-fresnel);
	float pi = 3.1415927;
	float n =  pow(1.5,gloss*10.0);
	return (pow(blinnTerm, n )*((n+8.0)/(8*pi)))*visibility;
}

float getnoise(vec2 pos) {
	return abs(fract(sin(dot(pos ,vec2(18.9898f,28.633f))) * 4378.5453f));
}

#ifdef CELSHADING
vec3 celshade(vec3 clrr) {
	//edge detect
	float d = edepth(newtc.xy);
	float dtresh = 1/(far-near)/5000.0;	
	vec4 dc = vec4(d,d,d,d);
	vec4 sa;
	vec4 sb;
	sa.x = edepth(newtc.xy + vec2(-pw,-ph)*1.0);
	sa.y = edepth(newtc.xy + vec2(pw,-ph)*1.0);
	sa.z = edepth(newtc.xy + vec2(-pw,0.0)*1.0);
	sa.w = edepth(newtc.xy + vec2(0.0,ph)*1.0);
	
	//opposite side samples
	sb.x = edepth(newtc.xy + vec2(pw,ph)*1.0);
	sb.y = edepth(newtc.xy + vec2(-pw,ph)*1.0);
	sb.z = edepth(newtc.xy + vec2(pw,0.0)*1.0);
	sb.w = edepth(newtc.xy + vec2(0.0,-ph)*1.0);
	
	vec4 dd = abs(2.0* dc - sa - sb) - dtresh;
	dd = vec4(step(dd.x,0.0),step(dd.y,0.0),step(dd.z,0.0),step(dd.w,0.0));
	
	float e = clamp(dot(dd,vec4(0.25f,0.25f,0.25f,0.25f)),0.0,1.0);
	return clrr*e;
}
#endif

float subSurfaceScattering(vec3 pos, float N) {
return pow(max(dot(lightVector,normalize(pos)),0.0),N)*(N+1)/6.28;

}

#ifdef WATER_REFRACT

float waterH(vec2 posxz) {

float wave = 0.0;


float factor = 1.0;
float amplitude = 0.15;
float speed = 3.5;
float size = 0.2;
	
posxz.x += sin(posxz.y+frameTimeCounter)*0.35;
posxz.y += cos(posxz.x+frameTimeCounter*0.5)*0.35;
	
float px = posxz.x/50.0 + 250.0;
float py = posxz.y/50.0  + 250.0;

float fpx = abs(fract(px*20.0)-0.5)*2.0;
float fpy = abs(fract(py*20.0)-0.5)*2.0;

float d = length(vec2(fpx,fpy));

for (int i = 1; i < 5; i++) {
wave -= d*factor*cos( (1/factor)*px*py*size + 1.0*frameTimeCounter*speed);
factor /= 2;
}

factor = 1.0;
px = -posxz.x/50.0 + 250.0;
py = -posxz.y/150.0 - 250.0;

fpx = abs(fract(px*20.0)-0.5)*2.0;
fpy = abs(fract(py*20.0)-0.5)*2.0;

d = length(vec2(fpx,fpy));
float wave2 = 0.0;
for (int i = 1; i < 5; i++) {
wave2 -= d*factor*cos( (1/factor)*px*py*size + 1.0*frameTimeCounter*speed);
factor /= 2;
}

return amplitude*wave2+amplitude*wave;
}

#endif

#ifdef WATER_CAUSTIC

vec3 waterCaustic(vec3 fposition,vec3 color) {

vec3 underwaterpos = vec3(texcoord.st, texture2D(depthtex1, texcoord.st).r);
underwaterpos = nvec3(gbufferProjectionInverse * nvec4(underwaterpos * 2.0 - 1.0));
vec4 worldpositionuw = gbufferModelViewInverse * vec4(underwaterpos,1.0);	
vec3 wpos = (worldpositionuw.xyz + cameraPosition.xyz);

wpos.xz += sin(frameTimeCounter+wpos.x*5.0 + wpos.z)*0.075;
wpos.xz += sin(wpos.z*5.0 + wpos.x)*0.075;

vec2 movement = vec2(frameTimeCounter/600,frameTimeCounter/600);
vec2 movement2 = vec2(-frameTimeCounter/600,frameTimeCounter/600);
vec2 movement3 = vec2(frameTimeCounter/600,-frameTimeCounter/600);
vec2 movement4 = vec2(-frameTimeCounter/600,-frameTimeCounter/600);

vec2 coord = (wpos.xz/460.0)+(movement/2);
vec2 coord1 = (wpos.xz/459.9)+(movement2/2);
vec2 coord2 = (wpos.xz/459.8)+(movement3/2);
vec2 coord3 = (wpos.xz/459.7)+(movement4/2);

float noise = texture2D(noisetex,fract(coord.xy/2.0)).x;
noise += texture2D(noisetex,fract(coord.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord.xy*2.0)).x/4.0;
noise += texture2D(noisetex,fract(coord.xy*4.0)).x/8.0;
noise += texture2D(noisetex,fract(coord1.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord1.xy*2.0)).x/4.0;
noise += texture2D(noisetex,fract(coord1.xy*4.0)).x/8.0;
noise += texture2D(noisetex,fract(coord2.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord2.xy*2.0)).x/4.0;
noise += texture2D(noisetex,fract(coord2.xy*4.0)).x/8.0;
noise += texture2D(noisetex,fract(coord3.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord3.xy*2.0)).x/4.0;
noise += texture2D(noisetex,fract(coord3.xy*4.0)).x/8.0;
float causticstrength = max((noise-1.6),0.0);
float wca = 0.2;
float caustic = (1.0 - (pow(wca,causticstrength)));
vec3 caustic_color = ivec3(0,0,0)/255.0/2.2;	//coloring caustics
vec3 wc = mix(color,caustic_color,caustic);

return wc;
}

vec3 underwaterCaustic(vec3 fposition,vec3 color) {

vec3 tpos = vec3(gbufferModelViewInverse * vec4(fposition,1.0));	
vec3 wpos = tpos.xyz + cameraPosition.xyz;

wpos.xz += sin(frameTimeCounter+wpos.x*5.0 + wpos.z)*0.075;
wpos.xz += sin(wpos.z*5.0 + wpos.x)*0.075;

vec2 movement = vec2(frameTimeCounter/600,frameTimeCounter/600);
vec2 movement2 = vec2(-frameTimeCounter/600,frameTimeCounter/600);
vec2 movement3 = vec2(frameTimeCounter/600,-frameTimeCounter/600);
vec2 movement4 = vec2(-frameTimeCounter/600,-frameTimeCounter/600);

vec2 coord = (wpos.xz/460.0)+(movement/2);
vec2 coord1 = (wpos.xz/459.9)+(movement2/2);
vec2 coord2 = (wpos.xz/459.8)+(movement3/2);
vec2 coord3 = (wpos.xz/459.7)+(movement4/2);

float noise = texture2D(noisetex,fract(coord.xy/2.0)).x;
noise += texture2D(noisetex,fract(coord.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord.xy*2.0)).x/4.0;
noise += texture2D(noisetex,fract(coord.xy*4.0)).x/8.0;
noise += texture2D(noisetex,fract(coord1.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord1.xy*2.0)).x/4.0;
noise += texture2D(noisetex,fract(coord1.xy*4.0)).x/8.0;
noise += texture2D(noisetex,fract(coord2.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord2.xy*2.0)).x/4.0;
noise += texture2D(noisetex,fract(coord2.xy*4.0)).x/8.0;
noise += texture2D(noisetex,fract(coord3.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord3.xy*2.0)).x/4.0;
noise += texture2D(noisetex,fract(coord3.xy*4.0)).x/8.0;
float causticstrength = max((noise-1.6),0.0);
float wca = 0.2;
float caustic = (1.0 - (pow(wca,causticstrength)));
vec3 caustic_color = ivec3(0,0,0)/255.0/2.2;	//coloring caustics
vec3 wc = mix(color,caustic_color,caustic);

return wc;
}

#endif

//////////////////////////////VOID MAIN//////////////////////////////

void main() {

#ifndef DYNAMIC_HANDLIGHT
		handlight = 0.0;
#endif
	vec2 newtc = texcoord.xy;
	//unpack material flags
	float land = float(aux.g > 0.04);
	float iswater = float(aux.g > 0.04 && aux.g < 0.07);
	float translucent = float(aux.g > 0.3 && aux.g < 0.5);
	float tallgrass = float(aux.g > 0.42 && aux.g < 0.48);
	float hand = float(aux.g > 0.75 && aux.g < 0.85);
	float emissive = float(aux.g > 0.58 && aux.g < 0.62);
	float shading = sky_lightmap;
#ifdef SHADOW
	shading = 0.0f;
#endif
	float second_shading = 0.0f;
	float spec = 0.0;
	
vec3 nsunlight = normalize(mix(pow(sunlight,vec3(2.2)),vec3(0.25,0.3,0.4),rainStrength));
sky_color = normalize(mix(sky_color,vec3(0.25,0.3,0.4),rainStrength)); //normalize colors in order to don't change luminance
	
	float fresnel_pow = 5.0;
	
	vec3 color = texture2D(gcolor, newtc.st).rgb;
	color = pow(color,vec3(2.2))*(1.0+translucent*0.3);
	
	float colLength = length(color);
	if (colLength > 0.5) colLength = 0.5+max(colLength-0.5,0.0)*.5;
	color = normalize(color)*colLength/sqrt(3.)*0.9;
	
	float NdotL = dot(lightVector,normal);
	float NdotUp = dot(normal,upVec);

	vec4 fragposition = gbufferProjectionInverse * vec4(newtc.s * 2.0f - 1.0f, newtc.t * 2.0f - 1.0f, 2.0f * pixeldepth - 1.0f, 1.0f);
	fragposition /= fragposition.w;
	
		vec4 worldposition = vec4(0.0);
		vec4 worldpositionraw = vec4(0.0);
		worldposition = gbufferModelViewInverse * fragposition;	
		float xzDistanceSquared = worldposition.x * worldposition.x + worldposition.z * worldposition.z;
		float yDistanceSquared  = worldposition.y * worldposition.y;
		worldpositionraw = worldposition;

	float time = float(worldTime);
	float transition_fading = 1.0-(clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-13500.0)/300.0,0.0,1.0) + clamp((time-22500.0)/300.0,0.0,1.0)-clamp((time-23400.0)/300.0,0.0,1.0));	//fading between sun/moon shadows
	float night = clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-22800.0)/200.0,0.0,1.0);
	
	vec3 uPos = vec3(.0);

	if (iswater > 0.9) {
	float uDepth = texture2D(depthtex1,newtc.xy).x;
	uPos  = nvec3(gbufferProjectionInverse * nvec4(vec3(newtc.xy,uDepth) * 2.0 - 1.0));	
	}
	
#ifdef WATER_REFRACT
	
	if (iswater > 0.9) {
	vec3 posxz = worldposition.xyz + cameraPosition.xyz;
	
	float deltaPos = 0.1;
	float h0 = waterH(posxz.xz);
	float h1 = waterH(posxz.xz + vec2(deltaPos,0.0));
	float h2 = waterH(posxz.xz + vec2(-deltaPos,0.0));
	float h3 = waterH(posxz.xz + vec2(0.0,deltaPos));
	float h4 = waterH(posxz.xz + vec2(0.0,-deltaPos));
	
	float xDelta = ((h1-h0)+(h0-h2))/deltaPos;
	float yDelta = ((h3-h0)+(h0-h4))/deltaPos;

	float refMult = 0.0065-dot(normal,normalize(fragposition).xyz)*0.001;
	
	vec3 refract = normalize(vec3(xDelta,yDelta,1.0-xDelta*xDelta-yDelta*yDelta));
	vec4 rA = texture2D(gcolor, newtc.st + refract.xy*refMult);
	rA.rgb = pow(rA.rgb,vec3(2.2));
	vec4 rB = texture2D(gcolor, newtc.st);
	rB.rgb = pow(rB.rgb,vec3(2.2));
	float mask = texture2D(gaux1, newtc.st + refract.xy*refMult).g;
	mask =  float(mask > 0.04 && mask < 0.07);
	newtc = (newtc.st + refract.xy*refMult)*mask + texcoord.xy*(1-mask);
	color.rgb = pow(texture2D(gcolor,newtc.xy).rgb,vec3(2.2));
	}
	
#endif
	
		float dist = length(fragposition.xyz);
		float distof = clamp(1.0-dist/shadowDistance,0.0,1.0);
		float distof2 = clamp(1.0-pow(dist/(shadowDistance*0.75),2.0),0.0,1.0);
		float shadow_fade = sqrt(clamp(1.0 - xzDistanceSquared / (shadowDistance*shadowDistance*1.0), 0.0, 1.0) * clamp(1.0 - yDistanceSquared / (shadowDistance*shadowDistance*1.0), 0.0, 1.0));

		/*--reprojecting into shadow space --*/

		worldposition = shadowModelView * worldposition;
		float comparedepth = -worldposition.z;
		float comparedepth2 = abs(worldposition.z);
		worldposition = shadowProjection * worldposition;
		worldposition /= worldposition.w;
	#ifdef SHADOW
		float distb = sqrt(worldposition.x * worldposition.x + worldposition.y * worldposition.y);
		float rescale = ((1.0f - SHADOW_MAP_BIAS) + distb * SHADOW_MAP_BIAS);
		float distortFactor = (1.0f - SHADOW_MAP_BIAS) + distb * SHADOW_MAP_BIAS;
		worldposition.xy *= 1.0f / distortFactor;
	#endif
		worldposition = worldposition * 0.5f + 0.5f;
	if (land > 0.9) {
		/*---------------------------------*/

	#ifdef SHADOW
		float step = 1.0/shadowMapResolution*(1.0+rainStrength*5.0);
		float NdotL = dot(normal, lightVector);
		float diffthresh = (pow(distortFactor*1.5,2.0)*(0.25/148.0)*(tan(acos(abs(NdotL)))) + (0.0/148.0))*(1.0+iswater*2.0);
		diffthresh = mix(diffthresh,0.0005,translucent);
	#endif

		if (comparedepth > 0.02 &&	worldposition.s < 0.98 && worldposition.s > 0.02 && worldposition.t < 0.98 && worldposition.t > 0.02 ) {
			if ((NdotL < 0.0 && translucent < 0.1) || (sky_lightmap < 0.01 && eyeBrightness.y < 2)) {
					shading = 0.0;
				}
			
	#ifdef SHADOW
	
			else {
			step = 0.5/shadowMapResolution*(1.0+rainStrength*5.0);
			
			#ifdef SHADOW_FILTER
				shading = shadow2D(shadow,vec3(worldposition.st, worldposition.z-diffthresh)).x;
				shading += shadow2D(shadow,vec3(worldposition.st + vec2(step,0), worldposition.z-diffthresh*2)).x;
				shading += shadow2D(shadow,vec3(worldposition.st + vec2(-step,0), worldposition.z-diffthresh*2)).x;
				shading += shadow2D(shadow,vec3(worldposition.st + vec2(0,step), worldposition.z-diffthresh*2)).x;
				shading += shadow2D(shadow,vec3(worldposition.st + vec2(0,-step), worldposition.z-diffthresh*2)).x;
				shading = shading/5.0;
			#endif

			#ifndef SHADOW_FILTER
				shading = shadow2D(shadow,vec3(worldposition.st, worldposition.z-diffthresh)).x;
			#endif 
			
			#ifdef WIP_VARIABLE_PENUMBRA_SHADOWS

				float avgdepth = .0;
				vec2 scales = vec2(0.,15.);
				float mult = 1.0;
				
				//using texture filtering instead of multiple samples for more sample coherence over pixels, plus huge performance improvement
				float ssample = comparedepth2 - (0.05 + (texture2DLod(shadowtex1, worldposition.st,1).z) * (256.0 - 0.05));
				avgdepth = clamp(ssample, scales.x, scales.y)/(scales.y);
							
				avgdepth = (avgdepth)*mult;
							
				diffthresh *= avgdepth+1.;			
				step =(0.15/shadowMapResolution*(1.+2.*tallgrass)+(avgdepth)/shadowMapResolution)/rescale*3.25*(1.0+rainStrength*5.0);
				float weight;
				
				for(int i = 0; i < 25; i++){
					float dist = length(shadow_offsets[i]);
					shading += shadow2D(shadow,vec3(worldposition.st + shadow_offsets[i]*step, worldposition.z-diffthresh*(1.0+dist))).x*exp(-dist*dist/0.3);
					weight += exp(-dist*dist/0.3);
				}

				
			shading /= weight;

			#endif
			
			}
		#endif
		}

		else shading = 1.0;

		if (sky_lightmap < 0.02 && eyeBrightness.y < 2) {
					shading = 0.0;
				}

float ao = 1.0;

//ssao config
const int nbdir = 6;				//the two numbers here affect the number of sample used. Increase for better quality at the cost of performance /6 and 6 is default
const float sampledir = 6;	
const float ssaorad = 10.0;			//radius of ssao shadows /1.0 is default

#ifdef SSAO
	
	if (land > 0.9 && iswater < 0.9 && hand < 0.9) {
	
	
		vec3 norm = texture2D(gnormal,texcoord.xy).rgb*2.0-1.0;
		vec3 projpos = convertScreenSpaceToWorldSpace(texcoord.xy,pixeldepth); 
		
		float progress = 0.0;
		ao = 0.0;
		
		float projrad = clamp(distance(convertCameraSpaceToScreenSpace(projpos + vec3(ssaorad,ssaorad,ssaorad)).xy,texcoord.xy),7.5*pw,60.0*pw);
		
		for (int i = 1; i < nbdir; i++) {
			for (int j = 1; j < sampledir; j++) {
				vec2 samplecoord = vec2(cos(progress),sin(progress))*(j/sampledir)*projrad + texcoord.xy;
				float sample = texture2D(depthtex0,samplecoord).x;
				vec3 sprojpos = convertScreenSpaceToWorldSpace(samplecoord,sample);
				float angle = pow(min(1.0-dot(norm,normalize(sprojpos-projpos)),1.0),2.0);
				float dist = pow(min(abs(ld(sample)-ld(pixeldepth)),0.015)/0.015,2.0);
				float temp = min(dist+angle,1.0);
				ao += pow(temp,3.0);
				//progress += (1.0-temp)/nbdir*3.14;
			}
			progress = i*1.256;
		}
		
		ao /= (nbdir-1)*(sampledir-1);

	}
	
#endif

		vec3 npos = normalize(fragposition.xyz);

		float diffuse = max(dot(lightVector,normal),0.0);
		
		diffuse = mix(diffuse,1.0,translucent*0.3);
		
		float sss = 1.0;
		
	#ifdef SHADOW
		sss = subSurfaceScattering(fragposition.xyz,30.0)*SUNLIGHTAMOUNT*2.0;
		sss = (mix(0.0,sss,max(shadow_fade-0.5,0.0)*2.0)*0.5+0.5)*translucent;
	#endif
		
		float handLight = (handlight*5.5)/pow(1.0+length(fragposition.xyz/2.4),2.0)*sqrt(dot(normalize(fragposition.xyz), -normal)*0.5+0.51);
		
	//Apply different lightmaps to image
		shading *= 1-isEyeInWater;
		
		float check_shadow = 0.8*sky_lightmap;
		
	#ifdef SHADOW
		check_shadow = 1.0;
	#endif
		
		vec3 light_col =  mix(pow(sunlight*check_shadow,vec3(2.2)),10*moonlight*check_shadow,moonVisibility);
		light_col = mix(light_col,vec3(length(light_col))*0.3,rainStrength*0.9);

		vec3 Sunlight_lightmap = light_col*shading*(1.0-rainStrength*0.95)*((((SUNLIGHTAMOUNT+sunVisibility*0.35)*eyeAdapt)+moonVisibility*(1-rainStrength))*(10.0-rainStrength*10.0)) *diffuse*transition_fading;

		vec3 Ucolor= normalize(vec3(0.0,0.13,0.2));

		
		//we'll suppose water plane have same height above pixel and at pixel water's surface
			//underwater position
		
		vec3 uVec = fragposition.xyz-uPos;
		float UNdotUP = abs(dot(normalize(uVec),normal));
		float depth = length(uVec)*UNdotUP;

	#ifdef FAKE_SHADOW
		if (isshaded < 0.99 && translucent == 0) color *= 0.7+(torch_lightmap+torch_lightmap2+rainStrength+moonVisibility)*0.3;
	#endif

		float NdotUp = (dot(normal,normalize(upPosition))*0.5+0.5);

		float visibility = sky_lightmap;
		float bouncefactor = sqrt((NdotUp*0.4+0.61) * pow(1.01-NdotL*NdotL,2.0)+0.5)*1.66+rainStrength*sunVisibility*4.0;
		float cfBounce = (-NdotL*0.45+0.56);
		
		vec3 emissive = vec3(3.0)*color.r*color.r*luma(color)*luma(color)*(emissive+handlight*hand)*eyeAdapt;

		vec3 bounceSunlight = 1.9*cfBounce*light_col*sky_lightmap*sky_lightmap*sky_lightmap*(SHADOW_DARKNESS-TimeMidnight*SHADOW_DARKNESS) * (1-rainStrength*0.9);
		
		float tL = (lightS.x*pow(sky_lightmap,2.2) + lightS.y)/5.5;
		float tLMoon = (lightS.z + lightS.w)/3.;
		
		vec3 skycolor = mix(sky_color, nsunlight,1-exp(-0.14*tL*(1-rainStrength*0.8)))*tL*sunVisibility*(1-rainStrength*0.8) + tLMoon*moonVisibility*moonlight*10;

		vec3 sky_light = ((SHADOW_DARKNESS-(SHADOW_DARKNESS-0.05)*rainStrength)*eyeAdapt)*skycolor*visibility*bouncefactor*(1+moonVisibility*0.9);

		vec3 torchcolor = vec3(TORCH_COLOR_LIGHTING)*eyeAdapt;

		vec3 Torchlight_lightmap = (torch_lightmap +torch_lightmap2+ handLight) *  torchcolor;
								 
		vec3 color_torchlight = Torchlight_lightmap;

	#ifdef WATER_CAUSTIC
		
		color += waterCaustic(fragposition.xyz,color.rgb)*iswater;
		color += underwaterCaustic(fragposition.xyz,color.rgb)*sky_lightmap*isEyeInWater;
		
	#endif

		color = ((bounceSunlight+sky_light)*ao+ Sunlight_lightmap*ao + color_torchlight*ao  +  sss * light_col * shading *(1.0-rainStrength*0.9)*transition_fading + emissive)*color;
		if (iswater > 0.9) color = mix(Ucolor*length(ambient_color)*0.002*sky_lightmap,color,exp(-depth/2));
		if (iswater > 0.9) color = mix(Ucolor*length(ambient_color)*0.025*sky_lightmap,color,exp(-depth/20));

		float gfactor = 0.8+(TimeNoon+TimeMidnight)*0.2;
		if (iswater < 0.9) spec = Blinn_Phong(fragposition.xyz,lightVector,normal,fresnel_pow,gfactor,shading*diffuse) *land * (1.0-isEyeInWater)*transition_fading;
		if (iswater > 0.9) spec = (Blinn_Phong(fragposition.xyz,lightVector,normal,fresnel_pow,2.0,shading*diffuse) *land * (1.0-isEyeInWater)*transition_fading);
	
	}

	else {
	color = pow(texture2D(gcolor,newtc.xy).rgb,vec3(2.2))*(1-sunVisibility)*1.0*sqrt(max(dot(upVec,normalize(fragposition.xyz)),0.0)) ;
	}
	
	//godrays config
	const float density = 0.7;			
	const int NUM_SAMPLES = 7;			//increase this for better quality at the cost of performance /5 is default
	const float grnoise = 1.;			//amount of noise /0.012 is default
	
float gr = 0.0;
	vec4 tpos = vec4(sunPosition,1.0)*gbufferProjection;
	tpos = vec4(tpos.xyz/tpos.w,1.0);
	vec2 pos1 = tpos.xy/tpos.z;
	vec2 lightPos = pos1*0.5+0.5;
	

		vec2 deltaTextCoord = vec2( newtc.st - lightPos.xy );
		vec2 textCoord = newtc.st;
		deltaTextCoord *= 1.0 /  float(NUM_SAMPLES) * density;
		float avgdecay = 0.0;
		float distx = abs(newtc.x*aspectRatio-lightPos.x*aspectRatio);
		float disty = abs(newtc.y-lightPos.y);
		float fallof = 1.0;
		float noise = getnoise(textCoord);
		
		for(int i=0; i < NUM_SAMPLES ; i++) {			
			textCoord -= deltaTextCoord;

			fallof *= 0.7;
			float sample = step(texture2D(gaux1, textCoord+ deltaTextCoord*noise*grnoise).g,0.01);
			gr += sample*fallof;
		}
	
#ifdef CELSHADING
	if (iswater < 0.9) color = celshade(color);
#endif
	color = clamp(pow(color/MAX_COLOR_RANGE,vec3(1.0/2.2)),0.0,1.0);

/* DRAWBUFFERS:31 */
	gl_FragData[0] = vec4(color, spec);
	gl_FragData[1] = vec4(vec3((gr/NUM_SAMPLES)),1.0);
}
