#version 120
#extension GL_ARB_shader_texture_lod : enable

/*
Before modify anything, read the agreement first that can be found here : https://werrusshaders.wordpress.com/download/
Werrus shaders, derived from Chocapic13 V5.
This code is from Chocapic13' shaders.
Disable an effect by putting "//" before "#define" when there is no number after.
*/

//////////////////////////////ADJUSTABLE VARIABLES//////////////////////////////

	#define GODRAYS						//enable godrays effect
		
	#define WATER_REFLECTIONS			//enable water reflection
	//#define LQ_WATER_REFLECTIONS		//water only reflect sky and sun
	
	#define SPECULAR_MAPPING 			//enable land reflection
	#define LQ_SPECULAR_MAPPING 		//land only reflect sky and sun
		
	#define CLOUDS						//enable clouds
		
	//#define NIGHT_STAR				//enable sparkling stars on night time sky

//////////////////////////////END OF ADJUSTABLE VARIABLES///////////////////////

const float 	wetnessHalflife 		= 50.0f;
const float 	drynessHalflife 		= 50.0f;

varying vec4 texcoord;

varying vec3 lightVector;
varying vec3 sunVec;
varying vec3 moonVec;
varying vec3 upVec;

varying vec3 sunlight;
varying vec3 moonlight;
varying vec3 ambient_color;

varying float eyeAdapt;

varying float SdotU;
varying float MdotU;
varying float sunVisibility;
varying float moonVisibility;

uniform sampler2D composite;
uniform sampler2D gaux1;
uniform sampler2D gaux3;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D gnormal;
uniform sampler2D gdepth;
uniform sampler2D noisetex;

uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform vec3 upPosition;
uniform vec3 cameraPosition;
uniform vec3 skyColor;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferModelView;

uniform ivec2 eyeBrightness;
uniform int isEyeInWater;
uniform int worldTime;
uniform float far;
uniform float near;
uniform float aspectRatio;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float wetness;
uniform float frameTimeCounter;
uniform int fogMode;
uniform ivec2 eyeBrightnessSmooth;
float timefract = worldTime;

#define MAX_COLOR_RANGE 48.0

//Calculate Time of Day
float TimeNoon     = ((clamp(timefract, 0.0, 4000.0)) / 4000.0) - ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0);

vec2 wind = vec2(frameTimeCounter/1000,0.0);
				
float pw = 1.0/ viewWidth;
float ph = 1.0/ viewHeight;

float matflag = texture2D(gaux1,texcoord.xy).g;

float pixeldepth = texture2D(depthtex0,texcoord.xy).x;
vec3 fragpos = vec3(texcoord.st, texture2D(depthtex0, texcoord.st).r);
vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0 - 1.0;

float time = float(worldTime);
float night = clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-22800.0)/200.0,0.0,1.0);
float night2 = clamp((time-13000.0)/300.0,0.0,1.0)-clamp((time-22700.0)/200.0,0.0,1.0);

float sky_lightmap = texture2D(gaux1,texcoord.xy).r;
float iswet = wetness*pow(sky_lightmap,5.0)*sqrt(0.5+max(dot(normal,upVec),0.0));

vec3 specular = pow(texture2D(gaux3,texcoord.xy).rgb,vec3(2.2));
float specmap = (specular.r+specular.g*(iswet));
	
vec4 color = texture2DLod(composite,texcoord.xy,0);


vec3 nvec3(vec4 pos) {
    return pos.xyz/pos.w;
}

vec4 nvec4(vec3 pos) {
    return vec4(pos.xyz, 1.0);
}

float cdist(vec2 coord) {
	return max(abs(coord.s-0.5),abs(coord.t-0.5))*2.0;
}


vec3 getSkyColor(vec3 fposition) {

vec3 sky_color = vec3(0.065, 0.25, 1.0);
vec3 nsunlight = normalize(pow(sunlight,vec3(2.2))*vec3(1.0,1.0,1.0));
vec3 sVector = normalize(fposition);

sky_color = normalize(mix(sky_color,vec3(0.25,0.3,0.4)*length(ambient_color),rainStrength)); //normalize colors in order to don't change luminance

float Lz = 0.5;
float cosT = dot(sVector,upVec); 
float absCosT = max(cosT,0.0);
float cosS = dot(sunVec,upVec);
float S = acos(cosS);				
float cosY = dot(sunVec,sVector);
float Y = acos(cosY);	

float a = -1.2;
float b = -0.4;
float c = 4.0;
float d = -0.6;
float e = 1.45;


//sun sky color
float L =  (1+a*exp(b/(absCosT+0.01)))*(1+(c+eyeAdapt*10.0)*exp(d*Y)+e*cosY*cosY); 
L = pow(L,1.0-rainStrength*0.8)*(1.0-rainStrength*0.83); //modulate intensity when raining

vec3 skyColorSun = mix(sky_color, nsunlight,1-exp(-0.005*pow(L,4.)*(1-rainStrength*0.5)))*L*0.5*vec3(1.0,1.0,1.0); //affect color based on luminance (0% physically accurate)
skyColorSun *= sunVisibility;


//moon sky color
float McosS = MdotU;
float MS = acos(McosS);
float McosY = dot(moonVec,sVector);
float MY = acos(McosY);

float L2 =  (1+a*exp(b/(absCosT+0.01)))*(1+c*exp(d*MY)+e*McosY*McosY)+0.2;
L2 = pow(L2,1.0-rainStrength*0.8)*(1.0-rainStrength*0.15); //modulate intensity when raining

vec3 skyColormoon = mix(moonlight,normalize(vec3(0.25,0.3,0.4))*length(moonlight),rainStrength*0.8)*L2 ; //affect color based on luminance (0% physically accurate)
skyColormoon *= moonVisibility;
sky_color = skyColormoon+skyColorSun;

return sky_color;
}


vec3 drawSun(vec3 fposition,vec3 color,int land) {
vec3 sVector = normalize(fposition);

float angle = (1-max(dot(sVector,sunVec),0.0))*350.0;
float sun = exp(-angle*angle);
sun *= land*(1-rainStrength*0.9925)*sunVisibility;
vec3 sunlight = mix(sunlight,vec3(0.25,0.3,0.4)*length(ambient_color),rainStrength*0.8);

return mix(color,sunlight*4.,sun);
}

vec3 skyGradient (vec3 fposition, vec3 color, vec3 fogclr) {
	return (fogclr*3.+color)/4.;
}

float getAirDensity (float h) {
return (max((h),60.0)-40.0)/2;
}

vec3 calcFog(vec3 fposition, vec3 color, vec3 fogclr) {
	float density = (18000.-rainStrength*10000) *(0.1+sunVisibility*0.1);

	vec3 worldpos = (gbufferModelViewInverse*vec4(fposition,1.0)).rgb+cameraPosition;
	float d = length(fposition);
	float height = mix(getAirDensity (worldpos.y)*2,0.1,rainStrength*0.1);

	float fog =   clamp(20.0*exp(-getAirDensity (cameraPosition.y)/density) * (1.0-exp( -d*height/density ))/height-0.3+rainStrength*0.25,0.0,1.);

return mix(color,normalize(fogclr)*mix(pow(length(fogclr),0.33)*sunlight,pow(length(fogclr),0.1)*vec3(0.09-moonVisibility*0.09),max(moonVisibility*(1-sunVisibility),rainStrength)),fog);	
}

float subSurfaceScattering(vec3 vec,vec3 pos, float N) {
return pow(max(dot(vec,normalize(pos)),0.0),N)*(N+1)/6.28;
}

float subSurfaceScattering2(vec3 vec,vec3 pos, float N) {
return pow(max(dot(vec,normalize(pos))*0.5+0.5,0.0),N)*(N+1)/6.28;
}

vec3 drawCloud(vec3 fposition,vec3 color) {

vec3 sVector = normalize(fposition);
float cosT = max(dot(normalize(sVector),upVec),0.0);
float McosY = MdotU;
float cosY = SdotU;
vec3 tpos = vec3(gbufferModelViewInverse * vec4(fposition,1.0));
vec3 wvec = normalize(tpos);
vec3 wVector = normalize(tpos);

vec4 totalcloud = vec4(.0);

vec3 intersection = wVector*((-300.0)/(wVector.y));
vec3 iSpos = (gbufferModelView*vec4(intersection,1.0)).rgb;
float cosT2 = pow(0.92,distance(vec2(0.0),intersection.xz)/100);
float ifenabled = 0.0;
#ifdef CLOUDS
ifenabled = 7.0;
#endif
for (int i = 0;i<ifenabled;i++) {
	intersection = wVector*((-cameraPosition.y+350.0-i*3*(1+cosT2*cosT2*3.5)+400*sqrt(cosT2))/(wVector.y)); 			//curved cloud plane
	vec3 wpos = tpos.xyz+cameraPosition;
	vec2 coord1 = (intersection.xz+cameraPosition.xz)/1000.0/180.+wind*0.035;
	vec2 coord2 = (intersection.xz+cameraPosition.xz)/1000.0/180.-wind*0.05;
	vec2 coord = fract(coord1/2.0);
	vec2 coord3 = fract(coord2/2.0);
	float noise = texture2D(noisetex,coord).x;	

	const float scale = 3;
	float mult = 1.0;
	float r = 2.3;
	float tmult = 1.1;

	for (int i = 0; i < 3; i++) {
	coord *= scale;
	mult /= r;
	noise += texture2D(noisetex,coord).x*mult;
	noise += texture2D(noisetex,coord3).x*mult;
	tmult += mult;
	}
	noise /= tmult;

	float cl = max(noise-(0.65-moonVisibility*0.1)  +wetness*0.3,0.0)*(1-wetness*0.4);
	float density = max(1-cl*0.5,0.)*max(1-cl*0.5,0.)*(i/7.)*(i/7.);
	  
	vec3 c = (24.*subSurfaceScattering(sunVec,fragpos,10.0)*pow(density,30.) + 10.*subSurfaceScattering2(sunVec,fragpos,1.1)*pow(density,2.5))*mix(sunlight,length(sunlight)*vec3(0.25,0.32,0.4),rainStrength)*sunVisibility +  (124.*subSurfaceScattering(moonVec,fragpos,3.1)*pow(density,30.) + 50.*subSurfaceScattering2(moonVec,fragpos,3.1)*pow(density,2.5))*mix(moonlight,length(moonlight)*vec3(0.25,0.32,0.4),rainStrength)*moonVisibility;
	cl = max(cl-(abs(i-8.0)/8.)*0.15,0.)*0.1;
	
	totalcloud += vec4(c.rgb*exp(-totalcloud.a),cl);
	totalcloud.a = min(totalcloud.a,1.0);
	
	if (totalcloud.a > 0.999) break;
}

return mix(color.rgb,totalcloud.rgb*(1 - rainStrength*0.92)*7.,totalcloud.a*pow(cosT2,1.2));

}

//raytrace config
const int maxf = 8;				//number of refinements
const float stp = 1.0;			//size of one step for raytracing algorithm
const float ref = 0.05;			//refinement multiplier
const float inc = 2.2;			//increasement factor at each step

vec4 raytrace(vec3 fragpos, vec3 normal,vec3 fogclr,vec3 sky_int) {
    vec4 color = vec4(0.0);
    vec3 start = fragpos;
    vec3 rvector = normalize(reflect(normalize(fragpos), normalize(normal)));
	
	//far black dots fix
	vec4 wrv = (gbufferModelViewInverse*vec4(rvector,1.0));
	wrv.y *= sign(dot(upVec,rvector));
	rvector = normalize((gbufferModelView*wrv).rgb);
	
    vec3 vector = stp * rvector;
    vec3 oldpos = fragpos;
    fragpos += vector;
	vec3 tvector = vector;
    int sr = 0;
	
    for(int i=0;i<40;i++){
        vec3 pos = nvec3(gbufferProjection * nvec4(fragpos)) * 0.5 + 0.5;
        if(pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1 || pos.z < 0 || pos.z > 1.0) break;
        vec3 spos = vec3(pos.st, texture2D(depthtex1, pos.st).r);
        spos = nvec3(gbufferProjectionInverse * nvec4(spos * 2.0 - 1.0));
        float err = abs(fragpos.z-spos.z);
		if(err < pow(length(vector)*2.0,1.15)){
                sr++;
                if(sr >= maxf){
                    float border = clamp(1.0 - pow(cdist(pos.st), 20.0), 0.0, 1.0);
                    color = texture2DLod(composite, pos.st,0);
					float land = texture2D(gaux1, pos.st).g;
					land = float(land < 0.03);
					
					spos.z = mix(spos.z,2000.0*(0.25+sunVisibility*0.75),land);
					if (land > 0.0) color.rgb = drawCloud(sky_int,skyGradient(sky_int,pow(color.rgb,vec3(2.2))*MAX_COLOR_RANGE,fogclr));
					else color.rgb = calcFog(spos,pow(color.rgb,vec3(2.2))*MAX_COLOR_RANGE,fogclr);
					color.a = 1.0;
                    color.a *= border;
                    break;
                }
				tvector -=vector;
                vector *=ref;
				
        
}

        vector *= inc;
        oldpos = fragpos;
        tvector += vector;
		fragpos = start + tvector;

    }
    return color;
}	

//land raytrace config
const int Gmaxf = 4;			//number of refinements
const float Gstp = 1.2;			//size of one step for raytracing algorithm
const float Gref = 0.06;		//refinement multiplier
const float Ginc = 3.0;			//increasement factor at each step

vec4 raytraceGround(vec3 fragpos, vec3 normal,vec3 fogclr) {
    vec4 color = vec4(0.0);
    vec3 start = fragpos;
    vec3 rvector = normalize(reflect(normalize(fragpos), normalize(normal)));
	
    vec3 vector = Gstp * rvector;
	
    vec3 oldpos = fragpos;
    fragpos += vector;
	vec3 tvector = vector;
    int sr = 0;
    for(int i=0;i<30;i++){
        vec3 pos = nvec3(gbufferProjection * nvec4(fragpos)) * 0.5 + 0.5;
		if(pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1 || pos.z < 0 || pos.z > 1.0) break;
        vec3 spos = vec3(pos.st, texture2D(depthtex1, pos.st).r);
        spos = nvec3(gbufferProjectionInverse * nvec4(spos * 2.0 - 1.0));
        float err = distance(fragpos.xyz,spos.xyz);
        if(err < pow(length(vector)*1.8,1.15)){

                sr++;
                if(sr >= maxf){
                    float border = clamp(1.0 - pow(cdist(pos.st), 20.0), 0.0, 1.0);
                    color = texture2DLod(composite, pos.st,0);
					float land = texture2D(gaux1, pos.st).g;
					land = float(land < 0.03);
					
					spos.z = mix(spos.z,2000.0*(0.25+sunVisibility*0.75),land);
					if (land > 0.0) color.rgb = skyGradient(spos,pow(color.rgb,vec3(2.2))*MAX_COLOR_RANGE,fogclr);
					else color.rgb = calcFog(spos,pow(color.rgb,vec3(2.2))*MAX_COLOR_RANGE,fogclr);
					color.a = 1.0;
                    color.a *= border;
                    break;
                }
				tvector -=vector;
                vector *=Gref;
				
        
}
        vector *= Ginc;
        oldpos = fragpos;
        tvector += vector;
		fragpos = start + tvector;
    }
    return color;
}

vec3 underwaterFog (float depth,vec3 color) {
	const float density = 48.0;
	float fog = exp(-depth/density);
	
	vec3 Ucolor= normalize(pow(vec3(0.0,0.1,0.15),vec3(2.2)))*(sqrt(3.0));
	
	vec3 c = mix(color*Ucolor,color,fog);
	vec3 fc = Ucolor*length(ambient_color)*0.02;
	return mix(fc,c,fog);
}

#ifdef NIGHT_STAR

vec3 nightStar(vec3 fposition,vec3 color) {
vec3 sVector = normalize(fposition);
float cosT = dot(sVector,upVec);
float McosY = MdotU;
float cosY = SdotU;
//star generation
/*----------*/
vec3 tpos = vec3(gbufferModelViewInverse * vec4(fposition,1.0));
vec3 wvec = normalize(tpos);
vec3 wVector = normalize(tpos);
vec3 intersection = wVector*(50.0/(wVector.y));

vec2 wind = vec2(abs(frameTimeCounter/1000.-0.5),abs(frameTimeCounter/1000.-0.5));

vec3 wpos = tpos.xyz+cameraPosition;
intersection.xz = intersection.xz + 5.0*cosT*intersection.xz;		//curve the star pattern, because sky is not 100% plane in reality
vec2 coord = (intersection.xz+wind*10)/512.0;
vec2 coord1 = (intersection.xz+wind*10)/512.0;
float noise = texture2D(noisetex,fract(coord.xy/2.0)).x;

float N = 8.0;
vec3 star_color = vec3(1.0, 1.0, 1.0)*1.0*moonVisibility*(1-rainStrength) + moonlight*48.0*pow(max(McosY,0.0),N)*(N+1)/6.28  * (1-rainStrength)*moonVisibility ;	//coloring stars
/*----------*/

noise += texture2D(noisetex,fract(coord.xy)).x/2.0;
noise += texture2D(noisetex,fract(coord1.xy)).x/2.0;

float cl = max(noise-1.7,0.0);
float ef = 0.1;

float star2 = (1.0 - (pow((1-rainStrength*0.9)*ef,cl)))*max(cosT,0.0);

vec3 s = mix(color,star_color,star2);

//s = mix(s,star_color,star);  //mix up sky color and stars

return s;
}

#endif

//////////////////////////////VOID MAIN//////////////////////////////

void main() {
	color.rgb = pow(color.rgb,vec3(2.2))*MAX_COLOR_RANGE;
	
	int land = int(matflag < 0.03);
	int iswater = int(matflag > 0.04 && matflag < 0.07);
	int hand  = int(matflag > 0.75 && matflag < 0.85);

	fragpos = nvec3(gbufferProjectionInverse * nvec4(fragpos * 2.0 - 1.0));
	float lightpos = max(dot(normalize(fragpos),lightVector),0.0);
	vec3 tfpos = fragpos.xyz;
	if (land > 0.9) fragpos = (gbufferModelView*(gbufferModelViewInverse*vec4(fragpos,1.0)+vec4(.0,max(cameraPosition.y-70.,.0),.0,.0))).rgb;
	vec3 uPos  = nvec3(gbufferProjectionInverse * nvec4(vec3(texcoord.xy,texture2D(depthtex1,texcoord.xy).x) * 2.0 - 1.0));		//underwater position
	float cosT = dot(normalize(fragpos),upVec);
	vec3 fogclr = getSkyColor(fragpos.xyz);
	uPos.z = mix(uPos.z,2000.0*(0.25+sunVisibility*0.75),land);
	
	float normalDotEye = clamp((dot(normal, normalize(fragpos))),-1.0,0.0);
	float fresnel = pow(1.0 + normalDotEye, mix(4.0+rainStrength,3.,iswater));
	fresnel = mix(1.,fresnel,0.95)*0.5;
		
	
		vec3 lc = mix(vec3(0.0),sunlight,sunVisibility);
		vec4 reflection = vec4(0.0);
		vec3 npos = normalize(fragpos);
		vec3 reflectedVector = reflect(normalize(fragpos), normalize(normal));
		float RdotU = (dot(reflectedVector,upVec)+1.)/2.;
		reflectedVector = fragpos + reflectedVector * (2000.0-fragpos.z);
		vec3 skyc = mix(getSkyColor(reflectedVector),vec3(0.002,0.005,0.002)*ambient_color*0.5,1-RdotU) ;
		vec3 sky_color = drawCloud(reflectedVector,skyGradient(reflectedVector,vec3(0.0),skyc)*clamp(sky_lightmap*2.0-2/16.0,0.0,1.0));
	
	if (iswater > 0.9) {

	#ifdef WATER_REFLECTIONS
		#ifndef LQ_WATER_REFLECTIONS
		reflection = raytrace(fragpos, normal,skyc,reflectedVector);
		#endif
		reflection.rgb = mix(sky_color, reflection.rgb, reflection.a)+(color.a)*lc*(1.0-rainStrength)*(5.+SdotU*45.);			
		reflection.a = min(reflection.a,1.0);
		reflection.rgb = (reflection.rgb*1.0);
		color.rgb = fresnel*reflection.rgb + (1-fresnel)*color.rgb;
	#endif
	
    }
else if (land < 0.9 && hand < 0.1) {


	#ifdef SPECULAR_MAPPING
		#ifndef LQ_SPECULAR_MAPPING
		if (specmap*fresnel > 0.005) reflection = raytraceGround(fragpos,normal,sky_color);
		#endif
		reflection.rgb = mix(sky_color, reflection.rgb, reflection.a)+(color.a)*lc*(1.0-rainStrength)*24.;		
		reflection.rgb = mix(reflection.rgb,reflection.rgb*normalize(color.rgb),0.0);
		reflection.rgb = reflection.rgb*3.;
		color.rgb = specmap*fresnel*reflection.rgb + (1-fresnel*specmap)*color.rgb;
	#endif

		}

	if (hand < 0.1) {
		if (land < 0.9) {
		if ((isEyeInWater == 1 && iswater > 0.9)|| (isEyeInWater == 0 && iswater < 0.9)) color.rgb = calcFog(uPos.xyz,color.rgb,(fogclr*clamp((eyeBrightness.y-25)/15.0,0.0,1.0)));
		else color.rgb = calcFog(fragpos.xyz,color.rgb,(fogclr*clamp((eyeBrightness.y-25)/15.0,0.0,1.0)));
		}
		else  {
		
	color.rgb = skyGradient(uPos.xyz,color.rgb,fogclr);
	color.rgb = drawSun(tfpos,color.rgb,land);

#ifdef NIGHT_STAR
	if (land > 0.9 && moonVisibility > 0.1) color.rgb = nightStar(tfpos.xyz,color.rgb);
#endif

	if (cosT > 0.) color.rgb = drawCloud(tfpos.xyz,color.rgb);
	}
	}
	if (isEyeInWater == 1) color.rgb = underwaterFog(length(fragpos),color.rgb);

	vec4 tpos = vec4(sunPosition,1.0)*gbufferProjection;
	tpos = vec4(tpos.xyz/tpos.w,1.0);
	vec2 pos1 = tpos.xy/tpos.z;
	vec2 lightPos = pos1*0.5+0.5;
	float gr = 0.0;
	
	const float exposure = 3.0;		//godrays intensity 15.0 is default
	const float density = 1.0;			
	const int NUM_SAMPLES = 7;		//increase this for better quality at the cost of performance /8 is default
	const float grnoise = 0.0;		//amount of noise /0.0 is default
	
#ifdef GODRAYS
	float truepos = sunPosition.z/abs(sunPosition.z);		//1 -> sun / -1 -> moon
	vec3 rainc = mix(vec3(1.),fogclr*0.5,rainStrength);
	vec3 lightColor = mix(sunlight*sunVisibility*rainc,10*moonlight*moonVisibility*rainc,(truepos+1.0)/2.);
	
	const int nSteps = NUM_SAMPLES;
	const float blurScale = 0.002/nSteps*9.0;
	const int center = (nSteps-1)/2;
	vec3 blur = vec3(0.0);
	float tw = 0.0;
	const float sigma = 0.5;
	
	vec2 deltaTextCoord = normalize(texcoord.st - lightPos.xy)*blurScale;
	vec2 textCoord = texcoord.st - deltaTextCoord*center;
	float distx = texcoord.x*aspectRatio-lightPos.x*aspectRatio;
	float disty = texcoord.y-lightPos.y;
	float illuminationDecay = pow(max(1.0-sqrt((distx*distx+disty*disty)*0.5),0.0),4.0);
	
		for(int i=0; i < nSteps ; i++) {
			textCoord += deltaTextCoord;
				
			float dist = (i-float(center))/center;
			float weight = exp(-(dist*dist)/(2.0*sigma));
				
			float sample = texture2D(gdepth, textCoord).r*weight;
			tw += weight;
			gr += sample;
		
		
		
	}
	vec3 grC = mix(lightColor,fogclr,rainStrength)*exposure*(gr/tw)*illuminationDecay * (1-isEyeInWater);
	color.xyz = (1-(1-color.xyz/48.0)*(1-grC.xyz/48.0))*48.0;

#endif
	
float visiblesun = 0.0;
float temp;
float nb = 0;
	
//calculate sun occlusion (only on one pixel) 
if (texcoord.x < 3.0*pw && texcoord.x < 3.0*ph) {
	for (int i = 0; i < 10;i++) {
		for (int j = 0; j < 10 ;j++) {
		temp = texture2D(gaux1,lightPos + vec2(pw*(i-5.0)*10.0,ph*(j-5.0)*10.0)).g;
		visiblesun +=  1.0-float(temp > 0.04) ;
		nb += 1;
		}
	}
	visiblesun /= nb;

}

	color.rgb = clamp(pow(color.rgb/MAX_COLOR_RANGE,vec3(1.0/2.2)),0.0,1.0);

/* DRAWBUFFERS:5 */
	gl_FragData[0] = vec4(color.rgb,visiblesun);
}