#version 120
#extension GL_ARB_shader_texture_lod : enable

/*
Before modify anything, read the agreement first that can be found here : https://werrusshaders.wordpress.com/download/
Werrus shaders, derived from Chocapic13 V5.
This code is from Chocapic13' shaders.
Disable an effect by putting "//" before "#define" when there is no number after.
*/

//////////////////////////////ADJUSTABLE VARIABLES//////////////////////////////

	#define VIGNETTE					//enable vignette effect
	#define VIGNETTE_STRENGTH 0.72		//default 0.72
	#define VIGNETTE_START 0.1			//distance from the center of the screen where the vignette effect start (0-1), default 0.1
	#define VIGNETTE_END 0.6			//distance from the center of the screen where the vignette effect end (0-1), bigger than VIGNETTE_START, default 0.7

	//#define SHAKING_CAMERA			//create shaking camera effect
	
	#define RAIN_DROPS					//create rain drops on screen

	//#define BLOOM						//enable bloom effect

//////////////////////////////END OF ADJUSTABLE VARIABLES///////////////////////

varying vec4 texcoord;

varying vec3 lightVector;
varying vec3 sunVec;
varying vec3 moonVec;
varying vec3 upVec;

varying vec3 sunlight;
varying vec3 moonlight;
varying vec3 ambient_color;

varying float eyeAdapt;

varying float SdotU;
varying float MdotU;
varying float sunVisibility;
varying float moonVisibility;

uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D depthtex2;
uniform sampler2D noisetex;
uniform sampler2D gnormal;
uniform sampler2D gdepth;
uniform sampler2D gaux1;
uniform sampler2D gaux2;
uniform sampler2D gaux4;
uniform sampler2D composite;
uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;
uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferPreviousProjection;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferPreviousModelView;
uniform ivec2 eyeBrightness;
uniform int isEyeInWater;
uniform int worldTime;
uniform float aspectRatio;
uniform float near;
uniform float far;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float wetness;
uniform float frameTimeCounter;
uniform int fogMode;
vec3 sunPos = sunPosition;
float pw = 1.0/ viewWidth;
float ph = 1.0/ viewHeight;
float timefract = worldTime;

#define MAX_COLOR_RANGE 48.0

//Raining
float rainx = clamp(rainStrength, 0.0f, 1.0f)/1.0f;
float wetx  = clamp(wetness, 0.0f, 1.0f);

//Calculate Time of Day
float TimeSunrise  = ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(timefract, 0.0, 4000.0)/4000.0));
float TimeNoon     = ((clamp(timefract, 0.0, 4000.0)) / 4000.0) - ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0);
float TimeSunset   = ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0) - ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0);
float TimeMidnight = ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0) - ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0);

float ld(float depth) {
    return (2.0 * near) / (far + near - depth * (far - near));
}

float luma(vec3 color) {
	return dot(color,vec3(0.299, 0.587, 0.114));
}

vec3 Uncharted2Tonemap(vec3 x) {
	float A = 0.4;							//brightness multiplier
	float B = 0.05;							//black level (lower means darker and more constrasted, higher make the image whiter and less constrasted)
	float C = 0.5;							//constrast level 
	float D = 0.2;		
	float E = 0.02;
	float F = 0.3;
	float W = MAX_COLOR_RANGE;
	return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F;
}

float distratio(vec2 pos, vec2 pos2) {
	float xvect = pos.x*aspectRatio-pos2.x*aspectRatio;
	float yvect = pos.y-pos2.y;
	return sqrt(xvect*xvect + yvect*yvect);
}

vec2 newtc = texcoord.xy;
vec3 aux = texture2D(gaux1, texcoord.st).rgb;

float smStep (float edge0,float edge1,float x) {
float t = clamp((x - edge0) / (edge1 - edge0), 0.0, 1.0);
return t * t * (3.0 - 2.0 * t); }

float waterDrop (vec2 tc) {
	vec2 drop = vec2(0.0,fract(frameTimeCounter/650.0));
	tc.x *= 10;
	float noise = texture2D(noisetex,(tc+drop)/2).x;
	noise += texture2D(noisetex,(tc+drop)).x/2;
	noise += texture2D(noisetex,(tc+drop)*2).x/4;
	noise += texture2D(noisetex,(tc+drop)*4).x/8;	
	noise += texture2D(noisetex,(tc+drop*0.5)).x/2;
	noise += texture2D(noisetex,(tc+drop*0.5)*2).x/4;
	noise += texture2D(noisetex,(tc+drop*0.5)*4).x/8;	
	float dropstrength = max(noise-1.8,0.0);
	float density = max(1-dropstrength*2.25,0.)*max(1-dropstrength*2.25,0.);
	float wdrop = 0.001;
	float waterD = (1.0 - (pow(wdrop,dropstrength)))*density;
	float waterD2 = (1.0 - (pow(wdrop,dropstrength)));
	waterD *= clamp((eyeBrightness.y-220)/15.0,0.0,1.0)*rainStrength;
	waterD2 *= clamp((eyeBrightness.y-220)/15.0,0.0,1.0)*rainStrength;
	float check_enabled = 0.0;
	#ifdef RAIN_DROPS
	check_enabled = 1.0;
	#endif
	return (waterD+waterD2*0.1)*check_enabled;
}

//////////////////////////////VOID MAIN//////////////////////////////

void main() {

	float iswater = float(aux.g > 0.04 && aux.g < 0.07);
	const float pi = 3.14159265359;

	vec2 fake_refract = vec2(sin(texcoord.x*5.0 + texcoord.y*5.0),cos(texcoord.y*5.0 + texcoord.x*5.0)) ;
	vec2 fake_refract2 = vec2(sin(frameTimeCounter*2 + texcoord.x*50.0 + texcoord.y*10.0),cos(frameTimeCounter*2 + texcoord.y*50.0 + texcoord.x*10.0)) ;

#ifdef SHAKING_CAMERA
	
	vec2 shaking_camera = vec2(sin(frameTimeCounter*1.3),cos(frameTimeCounter*2)) ;

#else

	vec2 shaking_camera = vec2(0.0, 0.0);

#endif

	vec2 newTC = texcoord.st + fake_refract * 0.05 * waterDrop(texcoord.xy/250) + fake_refract2 * 0.0025 * isEyeInWater + shaking_camera*0.002;

	vec3 color = pow(texture2D(gaux2, newTC).rgb,vec3(2.2))*MAX_COLOR_RANGE;

	float fog = 1-(exp(-pow(ld(texture2D(depthtex0, newTC.st).r)/256.0*far,4.0-rainStrength*2.2)*4.0));

#ifdef BLOOM
//Bloom
const float rMult = 0.0016;
const int nSteps = 15;

int center = (nSteps-1)/2;
float radius = center*rMult;

vec3 blur = vec3(0.0);
float tw = 0.0;

float sigma = 0.3;
float A = 1.0/sqrt(2.0*3.14159265359*sigma);


for (int i = 0; i < nSteps; i++) {

float dist = (i-float(center))/center;

float weight = A*exp(-(dist*dist)/(2.0*sigma));

blur += pow(texture2DLod(composite,texcoord.xy + rMult*vec2(1.0,aspectRatio)*vec2(0.0,i-center),2).rgb,vec3(2.2))*weight;

tw += weight;
}
blur /= tw;


color.rgb = mix(color,blur*MAX_COLOR_RANGE,(fog)*min(rainStrength+isEyeInWater,1.));

blur *= (2.5*eyeAdapt) * (sqrt(luma(blur)));
color.xyz = ((1-(1-color.xyz/48.0)*(1-blur.xyz))*48.0);

/*

color = ((blur + color/MAX_COLOR_RANGE) - (blur * color/MAX_COLOR_RANGE))*MAX_COLOR_RANGE;*/
#endif

	//Tonemapping
	vec3 curr = Uncharted2Tonemap(color);
	
	vec3 whiteScale = 1.0f/Uncharted2Tonemap(vec3(MAX_COLOR_RANGE));
	color = curr*whiteScale;

	#ifdef VIGNETTE
	float len = length(texcoord.xy-vec2(.5));
	float len2 = distratio(texcoord.xy,vec2(.5));
	float dc = mix(len,len2,0.3);
    float vignette = smStep(VIGNETTE_END, VIGNETTE_START,  dc);
	
	color = mix(color,color*vignette,VIGNETTE_STRENGTH);
	#endif

	color = clamp(pow(color,vec3(1.0/2.2)),0.0,1.0);

	gl_FragColor = vec4(color,1.0);
}
